@@ -1,10 +1,11 @@
-from inspect import isawaitable
 import logging
-import math
+from inspect import isawaitable
+
+import dask.config
 
-from .adaptive_core import AdaptiveCore
-from ..utils import log_errors, parse_timedelta
 from ..protocol import pickle
+from ..utils import log_errors, parse_timedelta
+from .adaptive_core import AdaptiveCore
 
 logger = logging.getLogger(__name__)
 
@@ -66,29 +67,45 @@ class Adaptive(AdaptiveCore):
 
     Notes
     -----
-    Subclasses can override :meth:`Adaptive.should_scale_up` and
+    Subclasses can override :meth:`Adaptive.target` and
     :meth:`Adaptive.workers_to_close` to control when the cluster should be
     resized. The default implementation checks if there are too many tasks
-    per worker or too little memory available (see :meth:`Adaptive.needs_cpu`
-    and :meth:`Adaptive.needs_memory`).
+    per worker or too little memory available (see
+    :meth:`Scheduler.adaptive_target`).
+    The values for interval, min, max, wait_count and target_duration can be
+    specified in the dask config under the distributed.adaptive key.
     '''
 
     def __init__(
         self,
         cluster=None,
-        interval="1s",
-        minimum=0,
-        maximum=math.inf,
-        wait_count=3,
-        target_duration="5s",
+        interval=None,
+        minimum=None,
+        maximum=None,
+        wait_count=None,
+        target_duration=None,
         worker_key=None,
         **kwargs
     ):
         self.cluster = cluster
         self.worker_key = worker_key
         self._workers_to_close_kwargs = kwargs
+
+        if interval is None:
+            interval = dask.config.get("distributed.adaptive.interval")
+        if minimum is None:
+            minimum = dask.config.get("distributed.adaptive.minimum")
+        if maximum is None:
+            maximum = dask.config.get("distributed.adaptive.maximum")
+        if wait_count is None:
+            wait_count = dask.config.get("distributed.adaptive.wait-count")
+        if target_duration is None:
+            target_duration = dask.config.get("distributed.adaptive.target-duration")
+
         self.target_duration = parse_timedelta(target_duration)
 
+        logger.info("Adaptive scaling started: minimum=%s maximum=%s", minimum, maximum)
+
         super().__init__(
             minimum=minimum, maximum=maximum, wait_count=wait_count, interval=interval
         )
@@ -110,6 +127,22 @@ class Adaptive(AdaptiveCore):
         return self.cluster.observed
 
     async def target(self):
+        """
+        Determine target number of workers that should exist.
+
+        Notes
+        -----
+        ``Adaptive.target`` dispatches to Scheduler.adaptive_target(),
+        but may be overridden in subclasses.
+
+        Returns
+        -------
+        Target number of workers
+
+        See Also
+        --------
+        Scheduler.adaptive_target
+        """
         return await self.scheduler.adaptive_target(
             target_duration=self.target_duration
         )
@@ -120,7 +153,7 @@ class Adaptive(AdaptiveCore):
             # are in sync before making recommendations.
             await self.cluster
 
-        return await super(Adaptive, self).recommendations(target)
+        return await super().recommendations(target)
 
     async def workers_to_close(self, target: int):
         """
