@@ -1,12 +1,14 @@
 import asyncio
+
 import pytest
 
-from dask.distributed import Worker, Scheduler, SpecCluster, Client
-from distributed.utils_test import slowinc, cleanup  # noqa: F401
+from dask.distributed import Client, Scheduler, SpecCluster, Worker
+
 from distributed.metrics import time
+from distributed.utils_test import cleanup, slowinc  # noqa: F401
 
 
-class SlowWorker(object):
+class SlowWorker:
     def __init__(self, *args, delay=0, **kwargs):
         self.worker = Worker(*args, **kwargs)
         self.delay = delay
