@@ -1,41 +1,41 @@
 import asyncio
-from contextlib import suppress
 import errno
 import logging
-from multiprocessing.queues import Empty
 import os
-import psutil
 import shutil
 import threading
 import uuid
 import warnings
 import weakref
+from contextlib import suppress
+from multiprocessing.queues import Empty
+
+import psutil
+from tornado import gen
+from tornado.ioloop import IOLoop, PeriodicCallback
 
 import dask
 from dask.system import CPU_COUNT
-from tornado.ioloop import IOLoop, PeriodicCallback
-from tornado import gen
 
+from . import preloading
 from .comm import get_address_host, unparse_host_port
 from .comm.addressing import address_from_user_args
-from .core import RPCClosed, CommClosedError, coerce_to_address, Status
+from .core import CommClosedError, RPCClosed, Status, coerce_to_address
 from .metrics import time
 from .node import ServerNode
-from . import preloading
 from .process import AsyncProcess
 from .proctitle import enable_proctitle_on_children
 from .security import Security
 from .utils import (
-    get_ip,
-    mp_context,
-    silence_logging,
-    json_load_robust,
-    parse_timedelta,
-    parse_ports,
     TimeoutError,
+    get_ip,
+    json_load_robust,
+    mp_context,
+    parse_ports,
+    parse_timedelta,
+    silence_logging,
 )
-from .worker import run, parse_memory_limit, Worker
-
+from .worker import Worker, parse_memory_limit, run
 
 logger = logging.getLogger(__name__)
 
@@ -144,7 +144,7 @@ class Nanny(ServerNode):
 
         self.Worker = Worker if worker_class is None else worker_class
         self.env = env or {}
-        self.config = config or {}
+        self.config = config or dask.config.config
         worker_kwargs.update(
             {
                 "port": worker_port,
@@ -168,7 +168,10 @@ class Nanny(ServerNode):
             local_directory = dask.config.get("temporary-directory") or os.getcwd()
             if not os.path.exists(local_directory):
                 os.makedirs(local_directory)
+            self._original_local_dir = local_directory
             local_directory = os.path.join(local_directory, "dask-worker-space")
+        else:
+            self._original_local_dir = local_directory
 
         self.local_directory = local_directory
 
@@ -198,7 +201,7 @@ class Nanny(ServerNode):
             "run": self.run,
         }
 
-        super(Nanny, self).__init__(
+        super().__init__(
             handlers=handlers, io_loop=self.loop, connection_args=self.connection_args
         )
 
@@ -333,7 +336,7 @@ class Nanny(ServerNode):
             worker_kwargs = dict(
                 scheduler_ip=self.scheduler_addr,
                 nthreads=self.nthreads,
-                local_directory=self.local_directory,
+                local_directory=self._original_local_dir,
                 services=self.services,
                 nanny=self.address,
                 name=self.name,
@@ -438,7 +441,7 @@ class Nanny(ServerNode):
         self.loop.add_callback(self._on_exit, exitcode)
 
     async def _on_exit(self, exitcode):
-        if self.status not in (Status.closing, Status.closed):
+        if self.status not in (Status.init, Status.closing, Status.closed):
             try:
                 await self.scheduler.unregister(address=self.worker_address)
             except (EnvironmentError, CommClosedError):
