@@ -1,18 +1,19 @@
 import asyncio
-from collections import defaultdict
 import logging
 import uuid
+from collections import defaultdict
 
-from .client import Future, Client
-from .utils import tokey, sync, thread_state
-from .worker import get_client
-from .utils import parse_timedelta
+from dask.utils import stringify
+
+from .client import Client, Future
+from .utils import parse_timedelta, sync, thread_state
+from .worker import get_client, get_worker
 
 logger = logging.getLogger(__name__)
 
 
 class QueueExtension:
-    """ An extension for the scheduler to manage queues
+    """An extension for the scheduler to manage queues
 
     This adds the following routes to the scheduler
 
@@ -44,7 +45,7 @@ class QueueExtension:
 
         self.scheduler.extensions["queues"] = self
 
-    def create(self, stream=None, name=None, client=None, maxsize=0):
+    def create(self, comm=None, name=None, client=None, maxsize=0):
         logger.debug("Queue name: {}".format(name))
         if name not in self.queues:
             self.queues[name] = asyncio.Queue(maxsize=maxsize)
@@ -52,7 +53,7 @@ class QueueExtension:
         else:
             self.client_refcount[name] += 1
 
-    def release(self, stream=None, name=None, client=None):
+    def release(self, comm=None, name=None, client=None):
         if name not in self.queues:
             return
 
@@ -66,7 +67,7 @@ class QueueExtension:
                 self.scheduler.client_releases_keys(keys=keys, client="queue-%s" % name)
 
     async def put(
-        self, stream=None, name=None, key=None, data=None, client=None, timeout=None
+        self, comm=None, name=None, key=None, data=None, client=None, timeout=None
     ):
         if key is not None:
             record = {"type": "Future", "value": key}
@@ -82,7 +83,7 @@ class QueueExtension:
             self.scheduler.client_releases_keys(keys=[key], client="queue-%s" % name)
             del self.future_refcount[name, key]
 
-    async def get(self, stream=None, name=None, client=None, timeout=None, batch=False):
+    async def get(self, comm=None, name=None, client=None, timeout=None, batch=False):
         def process(record):
             """ Add task status if known """
             if record["type"] == "Future":
@@ -122,12 +123,12 @@ class QueueExtension:
             record = process(record)
             return record
 
-    def qsize(self, stream=None, name=None, client=None):
+    def qsize(self, comm=None, name=None, client=None):
         return self.queues[name].qsize()
 
 
 class Queue:
-    """ Distributed Queue
+    """Distributed Queue
 
     This allows multiple clients to share futures or small bits of data between
     each other with a multi-producer/multi-consumer queue.  All metadata is
@@ -148,8 +149,8 @@ class Queue:
         Name used by other clients and the scheduler to identify the queue. If
         not given, a random name will be generated.
     client: Client (optional)
-        Client used for communication with the scheduler. Defaults to the
-        value of ``Client.current()``.
+        Client used for communication with the scheduler.
+        If not given, the default global client will be used.
     maxsize: int (optional)
         Number of items allowed in the queue. If 0 (the default), the queue
         size is unbounded.
@@ -168,7 +169,11 @@ class Queue:
     """
 
     def __init__(self, name=None, client=None, maxsize=0):
-        self.client = client or Client.current()
+        try:
+            self.client = client or Client.current()
+        except ValueError:
+            # Initialise new client
+            self.client = get_worker().client
         self.name = name or "queue-" + uuid.uuid4().hex
         self._event_started = asyncio.Event()
         if self.client.asynchronous or getattr(
@@ -201,7 +206,7 @@ class Queue:
     async def _put(self, value, timeout=None):
         if isinstance(value, Future):
             await self.client.scheduler.queue_put(
-                key=tokey(value.key), timeout=timeout, name=self.name
+                key=stringify(value.key), timeout=timeout, name=self.name
             )
         else:
             await self.client.scheduler.queue_put(
@@ -209,11 +214,11 @@ class Queue:
             )
 
     def put(self, value, timeout=None, **kwargs):
-        """ Put data into the queue
+        """Put data into the queue
 
         Parameters
         ----------
-        timeout: number or string or timedelta, optional
+        timeout : number or string or timedelta, optional
             Time in seconds to wait before timing out.
             Instead of number of seconds, it is also possible to specify
             a timedelta in string format, e.g. "200ms".
@@ -222,19 +227,19 @@ class Queue:
         return self.client.sync(self._put, value, timeout=timeout, **kwargs)
 
     def get(self, timeout=None, batch=False, **kwargs):
-        """ Get data from the queue
+        """Get data from the queue
 
         Parameters
         ----------
-        timeout: number or string or timedelta, optional
+        timeout : number or string or timedelta, optional
             Time in seconds to wait before timing out.
             Instead of number of seconds, it is also possible to specify
             a timedelta in string format, e.g. "200ms".
-        batch: boolean, int (optional)
+        batch : boolean, int (optional)
             If True then return all elements currently waiting in the queue.
             If an integer than return that many elements from the queue
             If False (default) then return one item at a time
-         """
+        """
         timeout = parse_timedelta(timeout)
         return self.client.sync(self._get, timeout=timeout, batch=batch, **kwargs)
 
